
#include <stdint.h>
#include <string.h>
#include "nrf_gpio.h"
#include "our_service.h"
#include "ble_srv_common.h"
#include "app_error.h"

/**@brief Function for handling the @ref BLE_GATTS_EVT_WRITE event from the S110 SoftDevice.
 *
 * @param[in] p_nus     Nordic UART Service structure.
 * @param[in] p_ble_evt Pointer to the event received from BLE stack.
 */
static void on_write(ble_os_t * p_our_service, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if (
        (p_evt_write->handle == p_our_service->char_handles.cccd_handle)
        &&
        (p_evt_write->len == 2)
       )
    {
        if (ble_srv_is_notification_enabled(p_evt_write->data))
        {
            p_our_service->is_notification_enabled = true;
        }
        else
        {
            p_our_service->is_notification_enabled = false;
        }
    }
    else if (
             (p_evt_write->handle == p_our_service->char_value_handles.value_handle)
             &&
             (p_our_service->data_handler != NULL)
            )
    {
        p_our_service->data_handler(p_our_service, p_evt_write->data, p_evt_write->len);
    }
    else
    {
        // Do Nothing. This event is not relevant for this service.
    }
}

// ALREADY_DONE_FOR_YOU: Declaration of a function that will take care of some housekeeping of ble connections related to our service and characteristic
void ble_our_service_on_ble_evt(ble_os_t * p_our_service, ble_evt_t * p_ble_evt)
{
    // OUR_JOB: Step 3.D Implement switch case handling BLE events related to our service. 
		switch (p_ble_evt->header.evt_id) 
		{
				case BLE_GAP_EVT_CONNECTED: 
					p_our_service->conn_handle = p_ble_evt->evt.gap_evt.conn_handle; 
					break; 
				case BLE_GAP_EVT_DISCONNECTED: 
					p_our_service->conn_handle = BLE_CONN_HANDLE_INVALID; 
					break; 
				case BLE_GATTS_EVT_WRITE:
            on_write(p_our_service, p_ble_evt);
            break;
				default: 
        // No implementation needed. 
        break; 
		}
}

/**@brief Function for adding RX characteristic.
 *
 * @param[in] p_nus       Nordic UART Service structure.
 * @param[in] p_nus_init  Information needed to initialize the service.
 *
 * @return NRF_SUCCESS on success, otherwise an error code.
 */
static uint32_t our_char_add(ble_os_t * p_our_service, const ble_os_init_t * p_os_init)
{
    	uint32_t            	err_code; 
			ble_uuid_t          	char_uuid;
			ble_uuid128_t       	base_uuid = BLE_UUID_OUR_BASE_UUID; 
			char_uuid.uuid      	= BLE_UUID_OUR_CHARACTERISTC_UUID;
			err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type); 
			APP_ERROR_CHECK(err_code);
	
		/**@snippet [Adding proprietary characteristic to S110 SoftDevice] */
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    //ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);

    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

   
    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &char_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = 8;//sizeof(uint8_t);
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = 20;
		uint8_t value[8]            = {0x12,0x34,0x56,0x78,0x00,0x00,0x00,0x00}; 
		attr_char_value.p_value     = value;

    err_code = sd_ble_gatts_characteristic_add(p_our_service->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_our_service->char_handles);
		APP_ERROR_CHECK(err_code);
		
    return NRF_SUCCESS;
    /**@snippet [Adding proprietary characteristic to S110 SoftDevice] */
}

/**@brief Function for adding our new characterstic to "Our service" that we initiated in the previous tutorial. 
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
/*static uint32_t our_char_add(ble_os_t * p_our_service)
{
    // OUR_JOB: Step 2.A, Add a custom characteristic UUID
	uint32_t            	err_code; 
	ble_uuid_t          	char_uuid;
	ble_uuid128_t       	base_uuid = BLE_UUID_OUR_BASE_UUID; 
	char_uuid.uuid      	= BLE_UUID_OUR_CHARACTERISTC_UUID;
	err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type); 
	APP_ERROR_CHECK(err_code);
	
	  // OUR_JOB: Step 2.F Add read/write properties to our characteristic
    ble_gatts_char_md_t char_md;
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read = 1; 
		//char_md.char_props.write = 1;
	
    
    // OUR_JOB: Step 3.A, Configuring Client Characteristic Configuration Descriptor metadata and add to char_md structure
    ble_gatts_attr_md_t cccd_md;
    memset(&cccd_md, 0, sizeof(cccd_md));
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm); 
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm); 
		cccd_md.vloc                = BLE_GATTS_VLOC_STACK;     
		char_md.p_cccd_md           = &cccd_md; 
		char_md.char_props.notify   = 1;
   
    
    // OUR_JOB: Step 2.B, Configure the attribute metadata
    ble_gatts_attr_md_t attr_md;
    memset(&attr_md, 0, sizeof(attr_md));
		attr_md.vloc        = BLE_GATTS_VLOC_STACK;		
    
    
    // OUR_JOB: Step 2.G, Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm); 
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // OUR_JOB: Step 2.C, Configure the characteristic value attribute
    ble_gatts_attr_t    attr_char_value;
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
		attr_char_value.p_attr_md   = &attr_md;
    
    // OUR_JOB: Step 2.H, Set characteristic length in number of bytes
		attr_char_value.max_len     = 8; 
		attr_char_value.init_len    = 8; 
		uint8_t value[8]            = {0x12,0x34,0x56,0x78,0x00,0x00,0x00,0x00}; 
		attr_char_value.p_value     = value;

    // OUR_JOB: Step 2.E, Add our new characteristic to the service
		err_code = sd_ble_gatts_characteristic_add(p_our_service->service_handle, 
                                   &char_md, 
                                   &attr_char_value, 
                                   &p_our_service->char_handles); 
		APP_ERROR_CHECK(err_code);
		
    return NRF_SUCCESS;
}
*/

/**@brief Function for adding TX characteristic.
 *
 * @param[in] p_nus       Nordic UART Service structure.
 * @param[in] p_nus_init  Information needed to initialize the service.
 *
 * @return NRF_SUCCESS on success, otherwise an error code.
 */
static uint32_t our_char_value_add(ble_os_t * p_our_service, const ble_os_init_t * p_os_init)
{
  	uint32_t            	err_code; 
		ble_uuid_t          	char_uuid;
		ble_uuid128_t       	base_uuid = BLE_UUID_OUR_BASE_UUID; 
		char_uuid.uuid      	= BLE_UUID_OUR_CHARACTERISTC_VALUE_UUID;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type); 
		APP_ERROR_CHECK(err_code);  
	
		ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    //ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.write         = 1;
    char_md.char_props.write_wo_resp = 1;
    char_md.p_char_user_desc         = NULL;
    char_md.p_char_pf                = NULL;
    char_md.p_user_desc_md           = NULL;
    char_md.p_cccd_md                = NULL;
    char_md.p_sccd_md                = NULL;


    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &char_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = 20;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = 20;

    return sd_ble_gatts_characteristic_add(p_our_service->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_our_service->char_value_handles);
}

/**@brief Function for adding our new characterstic to "Our service" that we initiated in the previous tutorial. 
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
/*static uint32_t our_char_value_add(ble_os_t * p_our_service)
{
    // OUR_JOB: Step 2.A, Add a custom characteristic UUID
	uint32_t            	err_code; 
	ble_uuid_t          	char_uuid;
	ble_uuid128_t       	base_uuid = BLE_UUID_OUR_BASE_UUID; 
	char_uuid.uuid      	= BLE_UUID_OUR_CHARACTERISTC_VALUE_UUID;
	err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type); 
	APP_ERROR_CHECK(err_code);
	
	  // OUR_JOB: Step 2.F Add read/write properties to our characteristic
    ble_gatts_char_md_t char_md;
    memset(&char_md, 0, sizeof(char_md));
		//char_md.char_props.read = 1; 
		char_md.char_props.write = 1;
	
    
    // OUR_JOB: Step 3.A, Configuring Client Characteristic Configuration Descriptor metadata and add to char_md structure
    ble_gatts_attr_md_t cccd_md;
    memset(&cccd_md, 0, sizeof(cccd_md));
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm); 
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm); 
		cccd_md.vloc                = BLE_GATTS_VLOC_STACK;     
		char_md.p_cccd_md           = &cccd_md; 
		char_md.char_props.notify   = 1;
   
    
    // OUR_JOB: Step 2.B, Configure the attribute metadata
    ble_gatts_attr_md_t attr_md;
    memset(&attr_md, 0, sizeof(attr_md));
		attr_md.vloc        = BLE_GATTS_VLOC_STACK;		
    
    
    // OUR_JOB: Step 2.G, Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm); 
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // OUR_JOB: Step 2.C, Configure the characteristic value attribute
    ble_gatts_attr_t    attr_char_value;
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
		attr_char_value.p_attr_md   = &attr_md;
    
    // OUR_JOB: Step 2.H, Set characteristic length in number of bytes
		attr_char_value.max_len     = 8; 
		attr_char_value.init_len    = 8; 
		uint8_t value[8]            = {0x12,0x34,0x56,0x78,0x00,0x00,0x00,0x00}; 
		attr_char_value.p_value     = value;

    // OUR_JOB: Step 2.E, Add our new characteristic to the service
		err_code = sd_ble_gatts_characteristic_add(p_our_service->service_handle, 
                                   &char_md, 
                                   &attr_char_value, 
                                   &p_our_service->char_value_handles); 
		APP_ERROR_CHECK(err_code);
		
    return NRF_SUCCESS;
}
*/
/**@brief Function for initiating our new service.
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
void our_service_init(ble_os_t * p_our_service, const ble_os_init_t * p_os_init)
{
    uint32_t   err_code; // Variable to hold return codes from library and softdevice functions

    // FROM_SERVICE_TUTORIAL: Declare 16-bit service and 128-bit base UUIDs and add them to the BLE stack
    ble_uuid_t        service_uuid;
    ble_uuid128_t     base_uuid = BLE_UUID_OUR_BASE_UUID;
    service_uuid.uuid = BLE_UUID_OUR_SERVICE_UUID;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
    APP_ERROR_CHECK(err_code);    
    
    // OUR_JOB: Step 3.B, Set our service connection handle to default value. I.e. an invalid handle since we are not yet in a connection.
		p_our_service->conn_handle = BLE_CONN_HANDLE_INVALID;
	
    // FROM_SERVICE_TUTORIAL: Add our service
    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
                                        &service_uuid,
                                        &p_our_service->service_handle);
    
    APP_ERROR_CHECK(err_code);
    
    // OUR_JOB: Call the function our_char_add() to add our new characteristic to the service. 
    our_char_add(p_our_service, p_os_init);
		our_char_value_add(p_our_service, p_os_init);
}

// ALREADY_DONE_FOR_YOU: Function to be called when updating characteristic value
void our_termperature_characteristic_update(ble_os_t *p_our_service, uint8_t *temperature_value)
{
    // OUR_JOB: Step 3.E, Update characteristic value
	if (p_our_service->conn_handle != BLE_CONN_HANDLE_INVALID) 
	{
			uint16_t               len = 8;
		ble_gatts_hvx_params_t hvx_params; 
		memset(&hvx_params, 0, sizeof(hvx_params)); 
 
		hvx_params.handle = p_our_service->char_handles.value_handle; 
		hvx_params.type   = BLE_GATT_HVX_NOTIFICATION; 
		hvx_params.offset = 0; 
		hvx_params.p_len  = &len; 
		hvx_params.p_data = (uint8_t *)temperature_value;
		sd_ble_gatts_hvx(p_our_service->conn_handle, &hvx_params);
 	}

}
